/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Implementation of ClassAdList class.

#include "trigger_common.h"

#include <iostream>
#include <fstream>
#include <strstream>

#include "ClassAdList.h"
#include "classad_triggers.h"

static const int DEBUG = 0;

//-------------------------------------------------------------------------
ClassAdList::ClassAdList()
{
	DIAG1_OUT("ClassAdList::ClassAdList()");
}

//-------------------------------------------------------------------------
ClassAdList::~ClassAdList()
{
	DIAG1_OUT("ClassAdList::~ClassAdList()");

	for ( ClassAdListIterator it = begin();it != end(); ++it ) {
		delete it->second;
	}
}

//-------------------------------------------------------------------------
void
ClassAdList::InsertAd(ClassAd *ad)
{
	DIAG2_OUT("ClassAdList::InsertAd()");

	string name;
	if ( !ad->EvaluateAttrString("Name", name) ) {
		// WARN_OUT("Ad has no \"Name\" attribute");
		name = "Unknown";
	}
	InsertAd(name, ad);
}

//-------------------------------------------------------------------------
void
ClassAdList::InsertAd(const string &name, ClassAd *ad)
{
	DIAG2_OUT("ClassAdList::InsertAd(" + name + ")");

	pair<string, ClassAd *> tmp(name, ad);
	classAds.insert(tmp);
}

//-------------------------------------------------------------------------
ErrorStatus
ClassAdList::FindAd(const string &name, ClassAd *&ad)
{
	DIAG2_OUT("ClassAdList::FindAd(" + name + ")");

	ErrorStatus		result;
	ad = NULL;

	pair<ClassAdListIterator, ClassAdListIterator>	its =
			classAds.equal_range(name);

	if ( its.first == its.second ) {
		// No ad with the given name.
			
	} else {
		// At least one ad with the given name.
		ad = its.first->second;

		if ( ++its.first != its.second ) {
			WARN_OUT("More than one ad with the same name");
			result += ErrorStatus(StatusWarn, WARN_DUPLICATE_NAME);
		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
ClassAdList::ReadFromFile(const string &fileName, bool isXml)
{
	DIAG1_OUT("ClassAdList::ReadFromFile(" + fileName + ")");

	ErrorStatus		result;

	if ( fileName == "-" ) {
		result += ReadFromStream(cin, isXml);
	} else {

		ifstream adsFile(fileName.c_str());

		if (!adsFile) {
			ERROR_OUT("Can't open ads file " + fileName + " for reading");
			result += ErrorStatus(StatusFailed, ERROR_FILE_IO);
		} else {
#if (__GNUC__ < 3)
			adsFile.flags(0);
#else
			adsFile.flags((std::_Ios_Fmtflags)0);
#endif
			result += ReadFromStream(adsFile, isXml);
		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
ClassAdList::WriteToFile(const string &fileName, bool isXml)
{
	DIAG1_OUT("ClassAdList::WriteToFile(" + fileName + ")");

	ErrorStatus result;

	if ( fileName == "-" ) {
		result += WriteToStream(cout, isXml);
	} else {

		ofstream adsFile(fileName.c_str());
	
		if ( !adsFile ) {
			ERROR_OUT("Can't open ads file for writing");
			result += ErrorStatus(StatusFailed, ERROR_FILE_IO);
		} else {
			result += WriteToStream(adsFile, isXml);
		}
	}

	return result;
}

//-------------------------------------------------------------------------
void
ClassAdList::MoveAds(ClassAdList &list)
{
	DIAG1_OUT("ClassAdList::MoveAds()");

	for ( ClassAdListIterator it = list.begin(); it != list.end(); ++it ) {
		ClassAd *ad = it->second;
		InsertAd(ad);
	}
	list.classAds.clear();
}

//-------------------------------------------------------------------------
ErrorStatus
ClassAdList::ReadFromAttr(const ClassAd *ad, const string& attrName)
{
	DIAG1_OUT("ClassAdList::ReadFromAttr(" + attrName + ")");

	ErrorStatus		result;

	ExprTree	*tree = ad->Lookup(attrName);
	if ( tree == NULL ) {
		ERROR_OUT("Attribute " + attrName + " not found");
		result += ErrorStatus(StatusFailed, ERROR_MISSING_ATTR);
	}
	
	if ( result.IsComplete() ) {
		if ( tree->GetKind() != ExprTree::EXPR_LIST_NODE ) {
			ERROR_OUT("Attribute " + attrName + " is not a list");
			result += ErrorStatus(StatusFailed, ERROR_ATTR_TYPE);
		}
	}

	if ( result.IsComplete() ) {
		ExprList	*list = (ExprList *)tree;
		for ( ExprList::iterator it = list->begin(); it < list->end(); ++it ) {
			ExprTree	*item = (ExprTree *)*it;
			if ( item->GetKind() == ExprTree::CLASSAD_NODE ) {
				ClassAd		*oldAd = (ClassAd *)item;
				ClassAd		*newAd = (ClassAd *)oldAd->Copy();
				InsertAd(newAd);
			}
		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
ClassAdList::ReadFromStream(istream &istr, bool isXml)
{
	DIAG2_OUT("ClassAdList::ReadFromStream()");

	ErrorStatus result;

	ClassAdParser		parser;
	ClassAdXMLParser	xmlParser;
	string				errorMsg;
	streampos			offset;

	while ( !istr.eof() ) {
		ClassAd *		ad;

		offset = istr.tellg();

		if ( isXml ) {
			ad = xmlParser.ParseClassAd(istr);
		} else {
			ad = parser.ParseClassAd(istr);
		}

		if ( ad ) {
			InsertAd(ad);
		} else {
				// This is kind of a kludgey way of figuring out whether
				// there's actually an error (vs. a blank line or something),
				// but it's apparently all we have.
			if ( CondorErrMsg != "" ) {
				errorMsg = CondorErrMsg;
				ostrstream		offsetMsg;
				offsetMsg << " at byte " << offset << ends;
				ERROR_OUT("Error parsing classad: " + errorMsg +
						offsetMsg.str());
				result += ErrorStatus(StatusFailed, ERROR_CLASSAD);
			}
		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
ClassAdList::WriteToStream(ostream &ostr, bool isXml)
{
	DIAG2_OUT("ClassAdList::WriteToStream()");

	ErrorStatus result;

	if ( isXml ) {
		ostr << "<?xml version=\"1.0\"?>\n";
		ostr << "<!DOCTYPE classads SYSTEM \"classads.dtd\">\n";
		ostr << "<classads>\n";
	}

	PrettyPrint			unParser;
	ClassAdXMLUnParser	xmlUnParser;

	for ( ClassAdListIterator it = begin(); it != end(); ++it ) {
		ClassAd *	ad = it->second;
		string		classAdStr;
		if ( isXml ) {
			xmlUnParser.SetCompactSpacing(false);
			xmlUnParser.Unparse(classAdStr, ad);
		} else {
			unParser.Unparse(classAdStr, ad);
		}
		ostr << classAdStr << endl;
	}

	if ( isXml ) {
		ostr << "</classads>\n";
	}

	return result;
}

/*============================================================================*/
