/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Declaration of ErrorStatus class.  This class implements status
// information to be returned by functions, etc.

#ifndef _ErrorStatus_h_
#define _ErrorStatus_h_

#include <sys/types.h>
#include <string>

// Status information (to be returned by a function, for example) and
// functions to get information about the status.  Note that the order
// here is significant for operator+=.
enum StatusVal {
	StatusInvalid = 0,
	StatusOk = 10000,	// Everything is OK.
	StatusWarn,			// There was an error, but the function completed.
	StatusCancel,		// The function was cancelled.
	StatusWarnCancel,	// There was a warning, and the function was cancelled.
	StatusFailed,		// Failure -- the function did not complete.
};

enum ErrorCode {
	ERROR_NONE = 0,
	ERROR_BAD_ARG,
	ERROR_FILE_IO,
	ERROR_CLASSAD,
	ERROR_MISSING_ATTR,
	ERROR_ATTR_TYPE,
	ERROR_BAD_CLASSAD,
	ERROR_SYSCALL,
	ERROR_CHILD,
	ERROR_INTERNAL_STATE,

	WARN_DUPLICATE_NAME,
};

class ErrorStatus
{
public:
	// -------------------------------------
	// Constructors and destructors.
	// -------------------------------------
	/**
	 * Default constructor.
	 */
	ErrorStatus() { _status = StatusOk; _ec = ERROR_NONE; }

	/**
	 * Constructor from StatusVal.
	 * @param The status for this object.
	 */
	ErrorStatus(StatusVal status) { _status = status; _ec = ERROR_NONE; }

	/**
	 * Constructor from StatusVal and ErrorCode.
	 * @param The status for this object.
	 * @param The error code for this object.
	 */
	ErrorStatus(StatusVal status, ErrorCode ec) { _status = status; _ec = ec; }

	/**
	 * Copy constructor.
	 */
	ErrorStatus(const ErrorStatus &other) {
			_status = other._status; _ec = other._ec; }

	/**
	 * Destructor.
	 */
	~ErrorStatus() {}

	// -------------------------------------
	// Accessors.
	// -------------------------------------
	/**
	 * Did the function complete successfully?
	 */
	bool IsComplete() const { return StatIsComplete(_status); }

	/**
	 * Was there an error?
	 * @return Whether this object indicates an error (not warning).
	 */
	bool IsError() const { return StatIsError(_status); }

	/**
	 * Was there a warning?
	 * @return Whether this object indicates a warning.
	 */
	bool IsWarn() const { return StatIsWarn(_status); }

	/**
	 * Was the function cancelled (for example, by the user)?
	 * @return Whether this object indicates cancellation.
	 */
	// Was the function cancelled (for example, by the user)?
	bool IsCancel() const { return StatIsCancel(_status); }

	/**
	 * Get the status value.
	 * @return The StatusVal of this object.
	 */
	StatusVal GetStatus() const { return _status; }

	/**
	 * Get the error code.
	 * @return The ErrorCode of this object.
	 */
	ErrorCode GetErrorCode() const { return _ec; }

	/**
	 * Get the status value of this object.
	 * @return The string equivalent of this object's status.
	 */
	string Value() const;

	// -------------------------------------
	// Assignment operators.
	// -------------------------------------
	/**
	 * Assignment of a StatusVal.
	 * @param The StatusVal to assign.
	 */
	void operator=(StatusVal status) { _status = status; }

	/**
	 * Assignment from another ErrorStatus object.
	 * @param The ErrorStatus object whose value will be assigned to this
	 * object.
	 */
	void operator=(const ErrorStatus &other) {
			_status = other._status; _ec = other._ec; }

	/**
	 * += on StatusVal.
	 * @param The StatusVal to be combined with the value of this object.
	 */
	void operator+=(StatusVal other) {
			_status = StatusCombine(_status, other); }

	/**
	 * += on ErrorStatus.
	 * @param The ErrorStatus object whose value is to be combined with
	 * the value of this object.
	 */
	void operator+=(const ErrorStatus &other);

	// -------------------------------------
	// I/O.
	// -------------------------------------
	/**
	 * Output to an ostream.
	 * @param The ostream to output to.
	 */
	friend ostream& operator<<(ostream& os, const ErrorStatus& status);

	// -------------------------------------
	// Misc. methods
	// -------------------------------------

	/**
	 * Convert an error (or canceled) status to a warning (this is used
	 * when a higher level can tolerate an error at a lower level).
	 */
	ErrorStatus &Error2Warning()
	{
		if ( _status == StatusCancel || _status == StatusWarnCancel ||
				_status == StatusFailed ) {
			_status = StatusWarn;
		}
		return *this;
	}

	/**
	 * Convert a canceled status to okay and cancel/warning to warning (this
	 * is used when a higher level can tolerate a cancellation at a lower
	 * level).
	 */
	ErrorStatus &Cancel2Ok()
	{
		if ( _status == StatusCancel ) {
			_status = StatusOk;
		} else if ( _status == StatusWarnCancel ) {
			_status = StatusWarn;
		}
		return *this;
	}

private:
	StatusVal	_status;
	ErrorCode	_ec;


	/**
	 * Whether the given status indicates completion.
	 */
	static bool StatIsComplete(StatusVal status) {
			return (status == StatusOk) || (status == StatusWarn); }

	/**
	 * Whether the given status indicates an error (does not include
	 * warnings).
	 */
	static bool StatIsError(StatusVal status) {
			return (status == StatusFailed); }

	/**
	 * Whether the given status indicates a warning.
	 */
	static bool StatIsWarn(StatusVal status) {
			return (status == StatusWarn) || (status == StatusWarnCancel); }

	/**
	 * Whether the given status indicates cancellation.
	 */
	static bool StatIsCancel(StatusVal status) {
			return (status == StatusCancel) || (status == StatusWarnCancel); }

	/**
	 * Combine two StatusVals.
	 * @param The first StatusVal.
	 * @param The second StatusVal.
	 * @return The StatusVal resulting from combining the inputs.
	 */
	static StatusVal StatusCombine(StatusVal status1, StatusVal status2);
};

#endif /* _ErrorStatus_h_ */

/*============================================================================*/
