/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Implementation of PreprocessStatus class.

// Note: this code assumes that all of the cvs lock information will be
// found in a single machine ad (or at least that the lock numbers will
// not conflict).

#include "trigger_common.h"

#include <iostream>
#include <strstream>
#include <vector>

#include "PreprocessStatus.h"
#include "TriggerUtil.h"

static const int DEBUG = 0;

//-------------------------------------------------------------------------
PreprocessStatus::PreprocessStatus()
{
	DIAG1_OUT("PreprocessStatus::PreprocessStatus()");
}

//-------------------------------------------------------------------------
PreprocessStatus::~PreprocessStatus()
{
	DIAG1_OUT("PreprocessStatus::~PreprocessStatus()");
}

//-------------------------------------------------------------------------
ErrorStatus
PreprocessStatus::Process(ClassAdList &statusAds,
		const ClassAdList &preProcAds)
{
	DIAG1_OUT("PreprocessStatus::Process()");

	ErrorStatus		result;

	ClassAdList		newStatusAds;

	for ( ConstClassAdListIterator status = statusAds.begin();
			status != statusAds.end(); ++status ) {
		const ClassAd *statusAd = status->second;
	 	for ( ConstClassAdListIterator preProc = preProcAds.begin();
				preProc != preProcAds.end(); ++preProc ) {
			const ClassAd *preProcAd = preProc->second;
			ErrorStatus tmpResult = ProcessPair(statusAd, preProcAd,
					newStatusAds);
			tmpResult.Cancel2Ok();
			result += tmpResult;
		}
	}

		// Now move the temporary ads into the main status ads list.
	statusAds.MoveAds(newStatusAds);

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
PreprocessStatus::ProcessPair(const ClassAd *statusAd,
		const ClassAd *preProcAd,
		ClassAdList &newStatusAds)
{
	DIAG2_OUT("PreprocessStatus::ProcessPair()");

	ErrorStatus		result;

		// Get the relevant count, if any.
	string	countAttrName;
	bool	gotCountAttr = false;
	int		count;
	if ( preProcAd->EvaluateAttrString("CountAttr", countAttrName) ) {
		gotCountAttr = true;
		if ( !statusAd->EvaluateAttrInt(countAttrName, count) ) {
			DIAG1_OUT("Unable to evaluate count attribute");
			result += ErrorStatus(StatusCancel, ERROR_MISSING_ATTR);
		}
	}

		// Get the relevant list, if any.
	string	listAttrName;
	bool	gotListAttr = false;
	string	list;
	if ( preProcAd->EvaluateAttrString("ListAttr", listAttrName) ) {
		gotListAttr = true;
		if ( !statusAd->EvaluateAttrString(listAttrName, list) ) {
			DIAG1_OUT("Unable to evaluate list attribute");
			result += ErrorStatus(StatusCancel, ERROR_MISSING_ATTR);
		}
	}

		// Make sure the pre-processing ad has a count attribute or a
		// list attribute defined, but not both.
	if ( gotCountAttr && gotListAttr ) {
		ERROR_OUT("Pre-processing ad may not have both CountAttr and "
				"ListAttr defined");
		result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
	} else if ( !gotCountAttr && !gotListAttr ) {
		ERROR_OUT("Pre-processing ad must have either CountAttr or "
				"ListAttr defined");
		result += ErrorStatus(StatusFailed, ERROR_MISSING_ATTR);
	}

	if ( result.IsComplete() ) {
		if ( gotCountAttr ) {
			result += ProcessPairCount(statusAd, preProcAd, count,
					newStatusAds);
		} else if ( gotListAttr ) {
			result += ProcessPairList(statusAd, preProcAd, list,
					newStatusAds);
		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
PreprocessStatus::ProcessPairCount(const ClassAd *statusAd,
		const ClassAd *preProcAd,
		int count, ClassAdList &newStatusAds)
{
	DIAG2_OUT("PreprocessStatus::ProcessPairCount()");

	ErrorStatus		result;

	ClassAdList		copyAttrs;
	copyAttrs.ReadFromAttr(preProcAd, "CopyAttrs");

	for ( int num = 0; num < count; ++num ) {
			// The "tag" differentiates this ad from other ads of the
			// same form.
		strstream	tagStream;
		tagStream << num << ends;
		string		tag = tagStream.str();

		result += CreateNewStatusAd(statusAd, preProcAd, copyAttrs, tag,
				newStatusAds);
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
PreprocessStatus::ProcessPairList(const ClassAd *statusAd,
		const ClassAd *preProcAd,
		const string &list, ClassAdList &newStatusAds)
{
	DIAG2_OUT("PreprocessStatus::ProcessPairList()");

	ErrorStatus		result;

	ClassAdList		copyAttrs;
	copyAttrs.ReadFromAttr(preProcAd, "CopyAttrs");

	vector<string *> tags;
	TriggerUtil::ParseString(list, ' ', tags);
	for ( int index = 0; index < tags.size(); ++index ) {
		string	*tag = tags[index];

		result += CreateNewStatusAd(statusAd, preProcAd, copyAttrs, *tag,
				newStatusAds);

		delete tag;
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
PreprocessStatus::CreateNewStatusAd(const ClassAd *statusAd,
		const ClassAd *preProcAd, const ClassAdList &copyAttrs,
		const string &tag, ClassAdList &newStatusAds)
{
	DIAG2_OUT("PreprocessStatus::CreateNewStatusAd(" + tag + ")");

	ErrorStatus		result;

		// Create a new ad to hold the information we need.
	ClassAd		*newAd = new ClassAd();

		// Copy the relevant attributes from the status ad to the
		// new ad, renaming them as necessary.
	result += CopyStatusAttrs(statusAd, copyAttrs, tag, newAd);

		// Give the new ad a name.  (Temporarily copy the "NewAdName"
		// expression into the new ad so that it can use attributes that
		// are in the new ad.)
	TriggerUtil::CopyAttr(preProcAd, "NewAdName", newAd);
	string		newAdName = GetNewAdName(newAd, tag);
	newAd->Delete("NewAdName");
	newAd->InsertAttr("Name", newAdName);

		// Insert the new ad into the list.
	newStatusAds.InsertAd(newAd);

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
PreprocessStatus::CopyStatusAttrs(const ClassAd *statusAd,
		const ClassAdList &copyAttrs,
		const string &tag, ClassAd *newAd)
{
	DIAG2_OUT("PreprocessStatus::CopyStatusAttrs(" + tag + ")");

	ErrorStatus		result;

	for ( ConstClassAdListIterator it = copyAttrs.begin();
			it != copyAttrs.end(); ++it ) {
		const ClassAd *ad = (ClassAd *)it->second;

		ErrorStatus		tmpResult;

		string		oldName;
		string		newName;

		if ( !ad->EvaluateAttrString("Old", oldName) ) {
			ERROR_OUT("\"Old\" attribute is missing from copy ad");
			tmpResult += ErrorStatus(StatusCancel, ERROR_MISSING_ATTR);
		} else if ( !ad->EvaluateAttrString("New", newName) ) {
			ERROR_OUT("\"New\" attribute is missing from copy ad");
			tmpResult += ErrorStatus(StatusCancel, ERROR_MISSING_ATTR);
		}

		if ( tmpResult.IsComplete() ) {
			string	oldName2 = TriggerUtil::SubstringReplace(oldName,
					"*", tag);
			tmpResult += TriggerUtil::CopyAttr(statusAd, oldName2,
					newAd, newName);
		}

		result += tmpResult;
	}

	return result;
}

//-------------------------------------------------------------------------
string
PreprocessStatus::GetNewAdName(const ClassAd *preProcAd, const string &tag)
{
	DIAG2_OUT("PreprocessStatus::GetNewAdName()");

	string		result;

	string		attrVal;
	if ( preProcAd->EvaluateAttrString("NewAdName", attrVal) ) {
		result = TriggerUtil::SubstringReplace(attrVal, "*", tag);
	} else {
		result = "Unknown";
	}

	return result;
}

/*============================================================================*/
