/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Declaration of PreprocessStatus class.  This class pre-processes
// status ads.  I eventually want to make it more flexible, but right
// now all it does is pull out the CVS lock info, and put that into its
// own ads.

#ifndef _PreprocessStatus_h_
#define _PreprocessStatus_h_

#include "ErrorStatus.h"
#include "Output.h"
#include "ClassAdList.h"

class PreprocessStatus
{
public:
	// -------------------------------------
	// Constructors and destructors.
	// -------------------------------------

	/** 
	 * Default constructor.
	 */
	PreprocessStatus();

	/** 
	 * Destructor.
	 */
	~PreprocessStatus();

	// -------------------------------------
	// Processing.
	// -------------------------------------

	/** 
	 * Process the given status ads.
	 * @param The list of status ads to process.
	 * @param The list of ads specifying the preprocessing.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus Process(ClassAdList &statusAds,
			const ClassAdList &preProcAds);

private:
	/**
	 * (Pre-)process a pair of ads (a status ad and a pre-processing ad).
	 * If the pre-processing ad applies to the status ad, this creates
	 * a new status ad with information extracted from the given status
	 * ad.
	 * @param The status ad to process.
	 * @param The pre-processing ad to process.
	 * @param The list of new ads to put the newly-created status ad into.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus ProcessPair(const ClassAd *statusAd, const ClassAd *preProcAd,
			ClassAdList &newStatusAds);

	/**
	 * (Pre-)process a pair of ads (a status ad and a pre-processing ad)
	 * where the pre-processing ad has CountAttr specified.
	 * @param The status ad to process.
	 * @param The pre-processing ad to process.
	 * @param The count (number of new status ads to create).
	 * @param The list of new ads to put the newly-created status ad into.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus ProcessPairCount(const ClassAd *statusAd,
			const ClassAd *preProcAd, int count, ClassAdList &newStatusAds);

	/**
	 * (Pre-)process a pair of ads (a status ad and a pre-processing ad)
	 * where the pre-processing ad has ListAttr specified.
	 * @param The status ad to process.
	 * @param The pre-processing ad to process.
	 * @param The list (list of new status ads to create).
	 * @param The list of new ads to put the newly-created status ad into.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus ProcessPairList(const ClassAd *statusAd,
			const ClassAd *preProcAd,
			const string &list, ClassAdList &newStatusAds);

	/**
	 * Create a new status ad with the status information from the given
	 * status ad, processed as specified in the given pre-processing ad.
	 * @param The status ad to process.
	 * @param The pre-processing ad to process.
	 * @param The list of attributes to copy from the given status ad to
	 *   the new status ad.
	 * @param The "tag" (string specifying a particular set of status info
	 *   in the status ad).
	 * @param The list of new ads to put the newly-created status ad into.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus CreateNewStatusAd(const ClassAd *statusAd,
			const ClassAd *preProcAd,
			const ClassAdList &copyAttrs, const string &tag,
			ClassAdList &newStatusAds);

	/**
	 * Copy appropriate status attributes from the given status ad to the
	 * new status ad, renaming them if necessary.
	 * @param The status ad to process.
	 * @param The list of attributes to copy from the given status ad to
	 *   the new status ad.
	 * @param The "tag" (string specifying a particular set of status info
	 *   in the status ad).
	 * @param The new status ad.
	 * @return ErrorStatus object giving the status of the operation.
	 */
	ErrorStatus CopyStatusAttrs(const ClassAd *statusAd,
			const ClassAdList &copyAttrs,
			const string &tag, ClassAd *newAd);

	/**
	 * Generate a name for the new ad, given the pre-processing ad and
	 * tag.
	 * @param The pre-processing ad.
	 * @param The "tag".
	 * @return The appropriate name for the new status ad.
	 */
	string GetNewAdName(const ClassAd *preProcAd, const string &tag);
};

#endif /* _PreprocessStatus_h_ */

/*============================================================================*/
