/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Implementation of ProblemProcess class.

#include "trigger_common.h"

#include <iostream>

#include "ProblemProcess.h"
#include "TriggerUtil.h"
#include "classad_triggers.h"

static const int DEBUG = 0;

//-------------------------------------------------------------------------
ProblemProcess::ProblemProcess()
{
	DIAG1_OUT("ProblemProcess::ProblemProcess()");

	string matchAdText =
			"[ state = [ other   = .trigger.ad;"
			"              ad      = [];"
			"            ];"
			"  trigger = [ other   = .state.ad;"
			"              ad      = [];"
			"            ];"
			"  ActionTrigger  = trigger.ad.ActionTrigger;"
			"  ActionText  = trigger.ad.ActionText;"
			"  Action  = trigger.ad.Action;"
			"]";

	ClassAdParser parser;

	matchAd = parser.ParseClassAd(matchAdText, true);
	if ( matchAd == NULL ) {
		ERROR_OUT("Can't create match classad");
		objectStatus += ErrorStatus(StatusFailed, ERROR_CLASSAD);
	}

	if ( objectStatus.IsComplete() ) {
		matchStateAd = (ClassAd *)matchAd->Lookup("state");
		matchTriggerAd = (ClassAd *)matchAd->Lookup("trigger");
		if ( matchStateAd == NULL || matchTriggerAd == NULL ) {
			ERROR_OUT("Can't get attributes from match classad");
			objectStatus += ErrorStatus(StatusFailed, ERROR_CLASSAD);
		}
	}
}

//-------------------------------------------------------------------------
ProblemProcess::~ProblemProcess()
{
	DIAG1_OUT("ProblemProcess::~ProblemProcess()");

	delete matchAd;
	matchAd = NULL;
	matchStateAd = NULL;
	matchTriggerAd = NULL;
}

//-------------------------------------------------------------------------
ErrorStatus
ProblemProcess::GenerateActionAds(/* const */ ClassAdList &stateAds,
		/* const */ ClassAdList &triggerAds, ClassAdList &actionAds)
{
	DIAG1_OUT("ProblemProcess::GenerateActionAds()");

	ErrorStatus		result;

	assert(actionAds.Size() == 0);

	for ( ClassAdListIterator state = stateAds.begin();
			state != stateAds.end(); state++ ) {

		for ( ClassAdListIterator trigger = triggerAds.begin();
				trigger != triggerAds.end(); trigger++ ) {
			ErrorStatus tmpResult = CheckAction(state->second,
					trigger->second, actionAds);
			if ( tmpResult.GetStatus() != StatusOk ) {
				result += ErrorStatus(StatusWarn, tmpResult.GetErrorCode());
			}

		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
ProblemProcess::ProcessActionAds(/* const */ ClassAdList &actionAds)
{
	DIAG1_OUT("ProblemProcess::ProcessActionAds()");

	ErrorStatus		result;

	for ( ClassAdListIterator it = actionAds.begin();
			it != actionAds.end(); ++it )
	{
		ClassAd		*ad = it->second;

		if ( DEBUG >= 3 ) {
			cout << "Action ad is " << *ad << endl;
		}

		string		actionText;
		bool gotActionText = ad->EvaluateAttrString("ActionText",
				actionText);

		string		actionType;
		bool gotActionType = false;
		ClassAd *actionAd = NULL;
		Value actionVal;
		bool gotAction = ad->EvaluateAttr("Action", actionVal);
		if ( gotAction ) {
			if ( actionVal.IsClassAdValue(actionAd) ) {
				gotActionType = actionAd->EvaluateAttrString("Type",
						actionType);
			}
		}
		if ( gotActionText && gotActionType ) {
			if ( actionType == "Mail" ) {

					// Get the list of email recipients.
				string		actionTo;
				vector<string *> values;
					// Get addresses to send email to.
				ErrorStatus tmpStatus = TriggerUtil::EvalStringOrList(
						actionAd, "To", values);
				result += tmpStatus;


					// Get the mail subject.
				string		subject;
				if ( !actionAd->EvaluateAttrString("Subject", subject) ) {
					subject = "";
				}

					// Send the email.
				if ( tmpStatus.IsComplete() ) {
					for (vector<string *>::iterator it = values.begin();
							it != values.end(); it++) {
						string *actionTo = *it;
						result += sm.CreateActionMail(*actionTo, subject,
								actionText);
					}
				} else {
					ERROR_OUT("Unable to get \"To\" value(s) in action ad");
					result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
				}
					// Delete the strings we got back.
				for (vector<string *>::iterator it = values.begin();
						it != values.end(); it++) {
					string *actionTo = *it;
					delete actionTo;
				}
				values.clear();
			
			} else {
				ERROR_OUT("Unsupported ActionType: " + actionType);
				result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
			}
		} else {
			ERROR_OUT("Action ad is missing ActionText or "
					"Action.Type");
			result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
		}
	}

	result += sm.SendAllMails();

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
ProblemProcess::CheckAction(/* const */ ClassAd *state,
		/* const */ ClassAd *trigger, ClassAdList &actionAds)
{
	DIAG2_OUT("ProblemProcess::CheckAction()");

	ErrorStatus		result;

	matchStateAd->Insert("ad", state);
	matchTriggerAd->Insert("ad", trigger);

	if ( DEBUG >= 3 ) {
		cout << "Match ad before flattening: <" << *matchAd << ">\n";
	}

	result += FlattenTriggerAd();

	if ( DEBUG >= 3 ) {
		cout << "Match ad after flattening: <" << *matchAd << ">\n";
	}

	if ( result.IsComplete() ) {
		bool foundMatch;
		if ( matchAd->EvaluateAttrBool("ActionTrigger", foundMatch) ) {
			if ( foundMatch ) {
				DIAG2_OUT("Found trigger match!");
	
				result += EvalActionAttrs(actionAds);
			}
		}
	}

	matchStateAd->Remove("ad");
	matchTriggerAd->Remove("ad");

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
ProblemProcess::FlattenTriggerAd()
{
	DIAG2_OUT("ProblemProcess::FlattenTriggerAd()");

	ErrorStatus		result;

	ExprTree *ad = matchTriggerAd->Lookup("ad");
	if ( ad ) {
		Value value;
		ExprTree *flattened;
		if ( matchAd->Flatten(ad, value, flattened) ) {
			if ( flattened &&
					(flattened->GetKind() == ExprTree::CLASSAD_NODE) ) {
				if ( DEBUG >= 3 ) {
					cout << "Flattened expression: <" <<
							*((ClassAd *)flattened) << ">\n";
				}
				matchTriggerAd->Remove("ad");
				matchTriggerAd->Insert("ad", flattened);
			} else {
				ERROR_OUT("trigger.ad did not flatten to a ClassAd");
				result += StatusFailed;
			}
		} else {
			ERROR_OUT("Flattening failed");
			result += StatusFailed;
		}
	} else {
		ERROR_OUT("Trigger ad doesn't contain \"ad\"");
		result += StatusFailed;
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
ProblemProcess::EvalActionAttrs(ClassAdList &actionAds)
{
	DIAG2_OUT("ProblemProcess::EvalActionAttrs()");

	ErrorStatus		result;

		// Get ActionText.
	string		actionText;
	bool gotActionText = matchAd->EvaluateAttrString("ActionText",
			actionText);

		// Get Action, and evaluate it (we're assuming it
		// will be a ClassAd).
	ClassAd *actionSubAd = NULL;
	Value actionVal;
	bool gotAction = matchAd->EvaluateAttr("Action", actionVal);
	if ( gotAction ) {
		if ( !actionVal.IsClassAdValue(actionSubAd) ) {
			ERROR_OUT("Trigger ad is not of the proper form ("
					"\"Action\" is not a ClassAd");
			result += ErrorStatus(StatusFailed, ERROR_BAD_CLASSAD);
		}
	}

	if ( gotActionText && actionSubAd ) {

		DIAG2_OUT("Action text: " + actionText);

			// Note: I'm not totally sure whether or not we should
			// be doing a copy here -- check into things further.
			// wenger@cs.wisc.edu, 2003-09-26.
		actionSubAd = (ClassAd *)actionSubAd->Copy();

		ClassAd		*actionParentAd = new ClassAd;
		bool insertedText = actionParentAd->InsertAttr("ActionText",
				actionText);
		bool insertedAction = actionParentAd->Insert("Action", actionSubAd);

		if ( insertedText && insertedAction ) {
			actionAds.InsertAd(actionParentAd);
		} else {
			ERROR_OUT("Unable to insert \"ActionText\" or "
					"\"Action\"");
			result += ErrorStatus(StatusFailed, ERROR_CLASSAD);
		}

	} else {

		ERROR_OUT("Trigger ad is not of the proper form ("
				"\"ActionText\" or \"Action\" "
				"attribute is missing)");
		result += ErrorStatus(StatusFailed, ERROR_MISSING_ATTR);
	}

	return result;
}

/*============================================================================*/
