/***************************Copyright-DO-NOT-REMOVE-THIS-LINE**
 * CONDOR Copyright Notice
 *
 * See LICENSE.TXT for additional notices and disclaimers.
 *
 * Copyright (c)1990-2004 CONDOR Team, Computer Sciences Department,
 * University of Wisconsin-Madison, Madison, WI.  All Rights Reserved.
 * No use of the CONDOR Software Program Source Code is authorized
 * without the express consent of the CONDOR Team.  For more information
 * contact: CONDOR Team, Attention: Professor Miron Livny,
 * 7367 Computer Sciences, 1210 W. Dayton St., Madison, WI 53706-1685,
 * (608) 262-0856 or miron@cs.wisc.edu.
 *
 * U.S. Government Rights Restrictions: Use, duplication, or disclosure
 * by the U.S. Government is subject to restrictions as set forth in
 * subparagraph (c)(1)(ii) of The Rights in Technical Data and Computer
 * Software clause at DFARS 252.227-7013 or subparagraphs (c)(1) and
 * (2) of Commercial Computer Software-Restricted Rights at 48 CFR
 * 52.227-19, as applicable, CONDOR Team, Attention: Professor Miron
 * Livny, 7367 Computer Sciences, 1210 W. Dayton St., Madison,
 * WI 53706-1685, (608) 262-0856 or miron@cs.wisc.edu.
****************************Copyright-DO-NOT-REMOVE-THIS-LINE**/

// Implementation of StateProcess class.

#include "trigger_common.h"

#include <iostream>

#include "StateProcess.h"
#include "classad_triggers.h"
#include "TriggerUtil.h"

static const int DEBUG = 0;

// Fixed strings used in the class ads.
static const char	*currentStateStr = "currentState";
static const char	*lastHeardFromStr = "LastHeardFrom";
static const char	*nameStr = "Name";
static const char	*previousHeardFromStr = "PreviousHeardFrom";
static const char	*previousStateStr = "previousState";
static const char	*stateChangeTimeStr = "StateChangeTime";

//-------------------------------------------------------------------------
StateProcess::StateProcess()
{
	DIAG1_OUT("StateProcess::StateProcess()");

	string matchAdText =
			"[ status = [ other   = .stateDef.ad;"
			"              my      = ad;"
			"              ad      = [];"
			"            ];"
			"  stateDef = [ other   = .status.ad;"
			"              my      = ad;"
			"              ad      = [];"
			"            ];"
			"  StateTrigger  = stateDef.ad.StateTrigger;"
			"  StateName	= stateDef.ad.StateName;"
			"  Name	= status.ad.Name;"
			"]";

	ClassAdParser parser;

	matchAd = parser.ParseClassAd(matchAdText, true);
	if ( matchAd == NULL ) {
		ERROR_OUT("Can't create match classad");
		objectStatus += ErrorStatus(StatusFailed, ERROR_CLASSAD);
	}

	if ( objectStatus.IsComplete() ) {
		matchStatusAd = (ClassAd *)matchAd->Lookup("status");
		matchStateDefAd = (ClassAd *)matchAd->Lookup("stateDef");
		if ( matchStatusAd == NULL || matchStateDefAd == NULL ) {
			ERROR_OUT("Can't get attributes from match classad");
			objectStatus += ErrorStatus(StatusFailed, ERROR_CLASSAD);
		}
	}
}

//-------------------------------------------------------------------------
StateProcess::~StateProcess()
{
	DIAG1_OUT("StateProcess::~StateProcess()");

	delete matchAd;
	matchAd = NULL;
	matchStatusAd = NULL;
	matchStateDefAd = NULL;
}

//-------------------------------------------------------------------------
ErrorStatus
StateProcess::GenerateStateAds(/* const */ ClassAdList &statusAds,
		/* const */ ClassAdList &stateDefAds,
		/* const */ ClassAdList &oldStateAds,
		ClassAdList &newStateAds)
{
	DIAG1_OUT("StateProcess::GenerateStateAds()");

	ErrorStatus		result;

	assert(newStateAds.Size() == 0);

	result += NewStateAds(statusAds, stateDefAds, newStateAds);

	if ( result.IsComplete() && oldStateAds.Size() > 0 ) {
		result += CombineStateAds(oldStateAds, newStateAds);
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
StateProcess::NewStateAds(/* const */ ClassAdList &statusAds,
		/* const */ ClassAdList &stateDefAds, ClassAdList &newStateAds)
{
	DIAG1_OUT("StateProcess::NewStateAds()");

	ErrorStatus		result;

	for ( ClassAdListIterator status = statusAds.begin();
			status != statusAds.end(); status++ ) {

		ClassAd		*state = new ClassAd();

		//
		// Put Name from the status ad into the new state ad.
		//
		string name;
		if ( status->second->EvaluateAttrString(nameStr, name) ) {
			state->InsertAttr(nameStr, name);
		} else {
			ERROR_OUT("No Name attribute found!");
			result += ErrorStatus(StatusFailed, ERROR_MISSING_ATTR);
		}

		//
		// Put LastHeardFrom from the status ad into the new state ad.
		//
		int lastHeardFrom;
		if ( status->second->EvaluateAttrInt(lastHeardFromStr,
				lastHeardFrom) ) {
			state->InsertAttr(lastHeardFromStr, lastHeardFrom);
		} else {
			ERROR_OUT("No LastHeardFrom attribute found!");
			result += ErrorStatus(StatusWarn, ERROR_MISSING_ATTR);
		}

		//
		// Go through all of the state definition ads, and add the
		// corresponding information to the new state ad.
		//
		for ( ClassAdListIterator stateDef = stateDefAds.begin();
				stateDef != stateDefAds.end(); stateDef++ ) {
			ErrorStatus tmpResult = CheckState(status->second,
					stateDef->second, state);
			if ( tmpResult.GetStatus() != StatusOk ) {
				result += ErrorStatus(StatusWarn, tmpResult.GetErrorCode());
			}
		}

		if ( result.IsComplete() ) {
			newStateAds.InsertAd(state);
		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
StateProcess::CombineStateAds(/* const */ ClassAdList &oldStateAds,
		ClassAdList &newStateAds)
{
	DIAG1_OUT("StateProcess::CombineStateAds()");

	ErrorStatus		result;

	//
	// For each entity ad in the new ads list, find the
	// corresponding ad in the old ads list, and combine the old state
	// with the new state, saving the combined state in the new ad.
	// (The "currentState" values from the old ad are put into the new
	// ad as "previousState" values.)
	//
	for ( ClassAdListIterator newIt = newStateAds.begin();
			newIt != newStateAds.end(); newIt++ ) {
		string newName;
		if ( !newIt->second->EvaluateAttrString(nameStr, newName) ) {
			ERROR_OUT("State ad is not of the proper form (no \"Name\" "
					"attribute");
			result += ErrorStatus(StatusWarn, ERROR_MISSING_ATTR);
			continue;
		}

		ClassAd		*oldAd;
		result += oldStateAds.FindAd(newName, oldAd);
		if ( oldAd != NULL ) {
			result += CombineEntityStates(oldAd, newIt->second);
		}
	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
StateProcess::CheckState(/* const */ ClassAd *status,
		/* const */ ClassAd *stateDef, ClassAd *stateAd)
{
	DIAG2_OUT("StateProcess::CheckState()");

	ErrorStatus		result;

	matchStatusAd->Insert("ad", status);
	matchStateDefAd->Insert("ad", stateDef);

	bool foundMatch;
	if ( matchAd->EvaluateAttrBool("StateTrigger", foundMatch) ) {

		string stateName;
		if ( stateDef->EvaluateAttrString("StateName", stateName) ) {
			ClassAd		*subState = new ClassAd();
			subState->InsertAttr(currentStateStr, foundMatch);

			stateAd->Insert(stateName, subState);

			result += CopyAttrs(status, stateDef, stateAd);
		} else {
			ERROR_OUT("No StateName attribute found!");
			result += ErrorStatus(StatusFailed, ERROR_MISSING_ATTR);
		}

	} else {
		// No-op -- this just means that the given state doesn't apply to
		// this status ad.
	}

	matchStatusAd->Remove("ad");
	matchStateDefAd->Remove("ad");

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
StateProcess::CopyAttrs(/* const */ ClassAd *status,
		/* const */ ClassAd *stateDef, ClassAd *stateAd)
{
	DIAG2_OUT("StateProcess::CopyAttrs()");

	ErrorStatus		result;

	ExprTree *attrs;
	attrs = stateDef->Lookup("CopyAttrs");
	if ( attrs ) {
		vector<string *> values;
		result += TriggerUtil::EvalStringOrList(stateDef, "CopyAttrs",
				values);
		if ( result.IsComplete() ) {
			for (vector<string *>::iterator it = values.begin();
					it != values.end(); it++) {
				string *attrName = *it;

				ExprTree *expr = status->Lookup(*attrName);
				if ( expr ) {
					stateAd->Insert(*attrName, expr->Copy());
				} else {
					WARN_OUT("Attribute " + *attrName +
							" is to be copied but is missing from status ad");
					result += StatusWarn;
				}
			}
		}


	}

	return result;
}

//-------------------------------------------------------------------------
ErrorStatus
StateProcess::CombineEntityStates(/* const */ ClassAd *oldState,
		ClassAd *newState)
{
	DIAG2_OUT("StateProcess::CombineEntityStates()");

	ErrorStatus		result;

	//
	// Find every attribute in the new state ad that's a "state sub ad"
	// (a nested class ad with a "currentState" attribute)...
	//
	ClassAdIterator newIt(*newState);
	string attr;
	const ExprTree *newExpr;

		//
		// Copy LastHeardFrom to PreviousHeardFrom.
		//
	int lastHeardFrom;
	if ( oldState->EvaluateAttrInt(lastHeardFromStr, lastHeardFrom) ) {
		newState->InsertAttr(previousHeardFromStr, lastHeardFrom);
	} else {
		// Not having the attribute is NOT an error here!
	}

	bool gotAttr = newIt.CurrentAttribute(attr, newExpr);
	while ( gotAttr ) {

		if ( IsStateSubAd(newExpr) ) {

			//
			// Okay, if we got to here we have a suitable sub ad from the
			// new state ad; now find the corresponding sub ad in the old
			// state ad...
			//
			ExprTree *oldExpr = oldState->Lookup(attr);
			if ( oldExpr != NULL && IsStateSubAd(oldExpr) ) {

				//
				// Now, copy the "currentState" value from the old sub ad, and
				// plug it into the new sub ad as "previousState".
				//
				ExprTree *oldVal = ((ClassAd *)oldExpr)->
						Lookup(currentStateStr);
				if ( oldVal != NULL ) {
					((ClassAd *)newExpr)->Insert(previousStateStr,
							oldVal->Copy());
				} else {
					ERROR_OUT("No current state value in old state sub ad");
					result += ErrorStatus(StatusFailed, ERROR_MISSING_ATTR);
				}

				//
				// Figure out whether the state changed this time.  If it
				// did change, plug the LastHeardFrom time into
				// StateChangeTime; if not, copy the StateChangeTime value
				// from the old state ad.
				//
				bool currentState;
				if ( !((ClassAd *)newExpr)->EvaluateAttrBool(currentStateStr,
						currentState) ) {
					ERROR_OUT("No current state value in new state sub ad");
					result += ErrorStatus(StatusFailed, ERROR_MISSING_ATTR);
				}

				bool prevState;
				if ( !((ClassAd *)newExpr)->EvaluateAttrBool(previousStateStr,
						prevState) ) {
					ERROR_OUT("No previous state value in new state sub ad");
					result += ErrorStatus(StatusFailed, ERROR_MISSING_ATTR);
				}

				if ( currentState != prevState ) {

						// State changed -- plug LastHeardFrom into this
						// state's StateChangeTime.
					int stateChangeTime;
					if ( newState->EvaluateAttrInt(lastHeardFromStr,
							stateChangeTime) ) {
						((ClassAd *)newExpr)->InsertAttr(stateChangeTimeStr,
								stateChangeTime);
					} else {
						ERROR_OUT("No last heard from value in new "
								"state sub ad");
						result += ErrorStatus(StatusFailed,
								ERROR_MISSING_ATTR);
					}
				} else {

						// State didn't change -- copy StateChangeTime from
						// old state ad.
					int stateChangeTime;
					if ( ((ClassAd *)oldExpr)->EvaluateAttrInt(
							stateChangeTimeStr, stateChangeTime) ) {
						((ClassAd *)newExpr)->InsertAttr(stateChangeTimeStr,
								stateChangeTime);
					} else {
						// Not having the attribute is NOT an error here!
					}
				}
			} else {
				ERROR_OUT("No old state sub ad corresponding to new state "
						"sub ad");
				result += ErrorStatus(StatusWarn, ERROR_MISSING_ATTR);
			}
		}

		gotAttr = newIt.NextAttribute(attr, newExpr);
	}

	return result;
}

//-------------------------------------------------------------------------
bool
StateProcess::IsStateSubAd(const ExprTree *tree)
{
	bool result = false;

	if ( tree->GetKind() == ExprTree::CLASSAD_NODE ) {
		ClassAd *ad = (ClassAd *)tree;
		if ( ad->Lookup(currentStateStr) != NULL ) {
			result = true;
		}
	}

	return result;
}

/*============================================================================*/
